define(['jquery', 'underscore', 'backbone'],
    function ($, _, Backbone) {

      var BaseModel = Backbone.Model.extend({
        defaultError: "Unexpected error.",
        save: function(attrs, options) {

          // check if user has supplied custom error callback
          if (options.error) {

            // save custom error callback
            var errorCallback = options.error;

            // get default error message
            var defaultError = this.defaultError;

            // create new error callback
            options.error = function (model, res, options) {

              var parsedErrors = {};

              // check if there is errors array/object
              if (res.responseJSON && res.responseJSON.errors) {
                parsedErrors.errors = res.responseJSON.errors;
              }

              // check if there is a single error
              if (res.responseJSON && res.responseJSON.error) {
                parsedErrors.error = res.responseJSON.error;
              }

              // if no errors were found in response, set default error message
              if (_.isEmpty(parsedErrors)) {
                parsedErrors.error = defaultError;
              }

              // call original error callback with parsed errors
              return errorCalback(model, res, options, parsedErrors);
            }
          }

          // call backbone model save method
          Backbone.Model.prototype.save.call(this, attrs, options);
        }
      });

      return BaseModel;
    });
