// Filename: router.js

define(["jquery", "underscore", "backbone", "views/content", "context", "events", "views/user/account", "cookiesjs"],
    function ($, _, Backbone, ContentView, Context, Events, AccountView, Cookies) {
        var AppRouter = Backbone.Router.extend({
            routes: {
                "": "index",
                "account": "account",
                "team": "team",
                "license": "license",
                "content/:query/:output/:connection(/:database)(/:table)": "content",
                "*actions": "default"
            }
        });

        var initialize = function () {

            var router = new AppRouter();

            router.on("route:index", function () {
                console.log("route:index", Context);

                var last_url = Cookies.get("last_url")
                if (!_.isEmpty(last_url)) {
                  // navigate to last url if any
                  this.navigate(last_url, {trigger: true, replace: true});
                } else {
                  // navigate to default test route
                  this.navigate("/content/filter/table/0", {trigger: true, replace: true});
                }
            });

            router.on("route:team", function () {
                console.log("route:team", Context);
                // team route
                Events.trigger("route:user:team");
            });

            router.on("route:account", function () {
                console.log("route:account", Context);
                // user account route
                Events.trigger("route:user:account");
            });

            router.on("route:license", function () {
                console.log("route:license", Context);
                // user account route
                Events.trigger("route:user:license");
            });

            router.on("route:content", function (query, output, connection, database, table) {

                // set context based on route params
                Context.connection = parseInt(connection);
                Context.db = database;
                Context.table = table;
                Context.query_mode = query;
                Context.output_mode = output;

                Events.trigger("route:content");
                //Events.trigger("connection:navigate", connection);

                console.log("route:content", Context);
            });

            router.on("route:default", function (actions) {
                console.log("default | not found", actions);
            });

            Backbone.history.syncURL = function () {

                var url;

                // @todo refactor and simplify this
                if (Context.table) {
                    url = "/content/%(query_mode)s/%(output_mode)s/%(connection)s/%(db)s/%(table)s".format(Context);
                } else if (Context.db) {
                    url = "/content/%(query_mode)s/%(output_mode)s/%(connection)s/%(db)s".format(Context);
                } else {
                    url = "/content/%(query_mode)s/%(output_mode)s/%(connection)s".format(Context);
                }

                //console.info("Backbone.history.syncURL", url, Context);
                this.navigate(url, {replace: true});
            };

            Backbone.history.getParam = function (key) {
                
                var parts = window.location.hash.split("?");
                
                if (parts.length < 2) {
                    return null;
                }

                var pairs = parts[1].split("&") || [];
                pairs.reduce(function (o, v) { o[v[0]] = v[1]; return o;}, {});

                if (!(key in pairs)) {
                    return null;
                }

                return pairs[key];
            };

            Backbone.history.start();

            return router;

        };
        return {
            initialize: initialize,
            router: AppRouter
        };
    });
