define([ 'jquery', 'underscore', 'backbone', 'tpls', 'views/content/output/viz', 'views/content/output/table', 'events',
        'views/content/output/viz/settings', 'baseview', 'context'],
    function ($, _, Backbone, Templates, VizView, TableView, Events, SettingsView, BaseView, Context) {
        return BaseView.extend({
            template: Templates["content/output"],
            events: {
                "click .content-menu .btn": "activate"
            },
            stopLoadingState: function () {
                this.$el.removeClass("loading");
            },
            startLoadingState: function () {
                this.$el.addClass("loading");
                this.resizePanel();
            },
            initialize: function () {
                this.listenTo(Events, "query:fetched error", this.stopLoadingState);
                this.listenTo(Events, "query:fetched", this.resizePanel);
                this.listenTo(Events, "query:fetching", this.startLoadingState); // table:activate

                _.bindAll(this, "resizePanel");

                // bind resize event to adjust max height of panel body
                $(window).resize(this.resizePanel);
            },
            resizePanel: function () {
                var resizeTimeout;
                var staticOffset = 7;
                var that = this;
                clearTimeout(resizeTimeout);
                resizeTimeout = setTimeout(function () {

                    // fit panel
                    var $panelBody = that.$(".panel-body");
                    var $grid = that.$(".sensei-grid-table-wrapper");
                    var po = $panelBody.offset();
                    var wh = $(window).height();
                    var maxHeight = wh - po.top - staticOffset;
                    $panelBody.css("max-height", maxHeight);

                    // fit sensei-grid
                    if ($grid.length > 0) {
                      var $toolbar = that.$(".sensei-grid-toolbar");
                      var gridMaxHeight = maxHeight - 15 - $grid.position().top - $toolbar.outerHeight();
                      $grid.css("max-height", gridMaxHeight);
                    }
                }, 50);
            },
            activate: function (e) {

                // set active item class and remove it from other elements
                $(e.target).addClass("active").siblings().removeClass("active");

                // set mode method name
                var mode_key = $(e.target).data("mode");
                var mode = mode_key + "Mode";

                // update url
                Context.output_mode = mode_key;
                Backbone.history.syncURL();

                if (this[mode]) {

                    // clear subviews
                    _.each(this.subviews, function (view) {
                        view.remove();
                    });

                    // activate mode method; it must set this.view
                    this[mode]();

                    if (Context.query_mode !== "raw") {
                      // trigger query:execute event
                      Events.trigger("query:execute");
                    }
                }
            },
            vizMode: function () {

                // initialize settings view
                this.subviews["settings"] = new SettingsView();
                this.subviews["settings"].parent = this;

                // initialize viz view
                this.subviews["viz"] = new VizView();
                this.subviews["viz"].parent = this;

                // render this.view
                this.$(".query-content").html(this.subviews["viz"].render().el);

                // render chart settings
                this.$(".chart-settings").html(this.subviews["settings"].render().el);
            },
            tableMode: function () {
                this.subviews["table"] = new TableView();

                // render this.view
                this.$(".query-content").html(this.subviews["table"].render().el);
            },
            render: function () {
                this.$el.html(this.template());

                // activate last output mode
                //this.$(".content-menu .btn:first").trigger("click");
                if (this.$(".content-menu .btn.active").length === 0) {
                    // currently output mode is not selected
                    //this.$(".content-menu .btn:first").addClass("active");
                    var el = this.$(".content-menu .btn").filter(function () {
                        return $(this).data("mode") === Context.output_mode;
                    }).get(0);

                    var e = {target: el};
                    this.activate(e);
                }

                // resize panel to set max height
                this.resizePanel();

                return this;
            }
        });
    });
