define(['jquery', 'underscore', 'backbone', 'collections/databases', 'context', 'views/sidebar/database', 'events',
        'tpls', 'models/connection'],
    function ($, _, Backbone, Databases, Context, DatabaseView, Events, Templates, Connection) {
        return Backbone.View.extend({
            template: Templates["sidebar/db"],
            initialize: function () {
                this.databases = new Databases();
                this.listenTo(this.databases, "request", function () {
                    // stop loading indicator
                    this.$(".active-db").html("");
                }, this);
                this.listenTo(this.databases, 'add', this.add);
                this.listenTo(Events, 'db:navigate', this.navigateDB);
                _.bindAll(this, "activateFirst", "error");
            },
            events: {
                "change .active-db": "changeDatabase"
            },
            navigateDB: function (name) {
                console.log("databases.navigateDB", name);
                this.$(".active-db option").filter(function () {
                    return $(this).val().trim() === name;
                }).prop("selected", true);
                this.$(".active-db").change();
            },
            changeDatabase: function (e) {

                // get db name from select box
                var db = $(e.target).val();

                console.log("changeDatabase", db, Context.db);

                // if (db !== Context.db) {
                    // reset table only if db has changed
                    Context.table = null;

                    // reset meta
                    Context.meta = {};

                    // reset query
                    Context.query = null;   

                    // set active db in global context
                    Context.db = db;

                    Backbone.history.syncURL();
                // }

                // trigger db:change event
                Events.trigger("db:change", db);

                // trigger query:execute event
                //Events.trigger("query:execute");
            },
            findDatabase: function (db) {
                return this.$(".active-db option").filter(function () {
                    return $(this).val() === db;
                });
            },
            activateFirst: function (model, data) {

                // store database list in context
                Context.dbs = data;

                // @todo refactor
                // set default db, should be refactored to gather this setting
                // from the params of active connection
                if (_.isEmpty(Context.db))  {
                    Context.db = "information_schema";
                }

                // find db item from active db in context
                var $dbItem = this.findDatabase(Context.db);


                if (Context.db === null || $dbItem.length === 0) {
                    this.$(".active-db option:first").attr("selected", true);
                } else {
                    $dbItem.attr("selected", true);
                }
                var activeItemLength = this.$(".active-db option:selected").length;

                if (activeItemLength) {
                    // trigger db change if there is an active db
                    Events.trigger("db:change", this.$(".active-db").val());
                } else {
                    console.warn("couldn't select any database", Context, $dbItem);
                }

                console.log("DatabasesView.activateFirst", Context.db, $dbItem, activeItemLength);

                Backbone.history.syncURL();
            },
            add: function (db) {
                var view = new DatabaseView({model: db});
                this.$(".active-db").append(view.render().el);
            },
            render: function () {
                this.$el.html(this.template());

                // start loading indicator
                this.$(".active-db").html($("<option>").text("Loading databases..."));

                this.databases.fetch({success: this.activateFirst, error: this.error});
                return this;
            },
            error: function (model, error) {

                // set context as if it were loaded
                Context.loaded = true;

                Events.trigger("error", "db", error["responseJSON"]);
                if (typeof error === "string") {
                    // error is passed as a string, output it
                    alert(error);
                } else if (error["responseJSON"] && error["responseJSON"]["error"]) {

                    alert("Error: %(error)s".format(error["responseJSON"]));

                    // if there is a problem with database connection, pass the event to main view
                    // which will call editConnection view in response
                    if (error["responseJSON"]["code"] === "0001") {
                        // get all connections
                        var conns = Context.connections.connections;
                        // try to find connection data in connection list
                        var conn = _.findWhere(conns, {id: Context.connection});
                        // create model of current connection
                        var connModel = new Connection(conn);
                        // trigger error
                        Events.trigger("connection:error", connModel, error);
                    }
                } else {
                    alert("Unexpected error, please try to reload the page.");
                }
            }
        });
    });
